import json

dependency_list = []


def get_dependencies(data, parent=''):
    for dependency, depdata in data.get('dependencies').items():
        item = {'package': dependency, 'version': depdata.get('version')}
        if parent:
            item['parent'] = '{}@{}'.format(parent, data.get('version'))
        if 'dependencies' in depdata:
            get_dependencies(depdata, dependency)
        dependency_list.append(item)


def generate_tree(filepath):
    with open(filepath) as f:
        data = json.loads(f.read())
    get_dependencies(data)
    appname = {'package': data.get('name'), 'version': data.get('version')}
    return dependency_list, appname



from zasca import utils, scanner
from zasca.maven_scanner import maven_tree_generator
from zasca.node_scanner import node_tree_generator
from tqdm import tqdm
import collections
import sys
import click

EMPTY_SUPPRESSION = 'emptysuppression.json'


def scan_maven(filepath, include_dev):
    maven_tree_generator.generate_tree(filepath, include_dev)
    dependencies, appname = maven_tree_generator.get_dependencies()
    mavenscan = scanner.Scanner(appname)
    trigger_scan(dependencies, mavenscan, 'MAVEN')
    return mavenscan.advisory_list, mavenscan.appname, dependencies


def scan_node(filepath):
    dependencies, appname = node_tree_generator.generate_tree(filepath)
    nodescan = scanner.Scanner(appname)
    trigger_scan(dependencies, nodescan, 'NPM')
    return nodescan.advisory_list, nodescan.appname, dependencies


def trigger_scan(dependencies, scanner, ecosystem):
    print("Scanning dependencies...")
    for dependency in tqdm(dependencies[0:3]):
        advisories = scanner.get_advisories(dependency.get('package'), ecosystem)
        if advisories:
            scanner.validate_vulnerable_version(advisories, dependency.get('package'), dependency.get('version'))


def write_output(num_issues, unique_libraries, num_fp, qg):
    print('{} vulnerabilities detected in {} vulnerable libraries'.format(num_issues, unique_libraries))
    if num_fp:
        print('{} vulnerabilities supressed'.format(num_fp))
    print('Quality gate passed: {}'.format(qg))


@click.command()
@click.argument('file', required=True)
@click.option('--sbom', help='Generates CycloneDX SBOM', default=True)
@click.option('--include_dev', help='Include dev dependencies', default=False)
@click.option('--quality_gate', help='Maximum severity allowed', default='LOW')
@click.option('--suppression_file', help='False positives to remove', default=EMPTY_SUPPRESSION)
def run_cli(file, sbom, include_dev, quality_gate, suppression_file):
    suppressed_items = []
    if file == 'pom.xml':
        data, appname, dependencies = scan_maven(file, include_dev)
    if file == 'package-lock.json':
        data, appname, dependencies = scan_node(file)
    if sbom:
        utils.generate_cyclonedx_sbom(dependencies)
    if suppression_file != EMPTY_SUPPRESSION:
        maven_data, suppressed_items = utils.suppress_fp(data, suppression_file)
    utils.generate_html_report(data, appname)
    unique_vuln_libraries = collections.Counter(item['package'] for item in data)
    severity_data = collections.Counter(item.get('advisory').get('severity') for item in data)
    qg_passed = utils.check_quality_gate(severity_data, quality_gate)
    write_output(len(data), len(unique_vuln_libraries), len(suppressed_items), qg_passed)
    sys.exit(not qg_passed)



import requests
import json
import yaml
import os

headers = {"Authorization": "bearer {}".format(os.getenv("GITHUB_TOKEN"))}

PATH_PROJECT = os.path.realpath(os.path.dirname(__file__))


class Scanner():
    def __init__(self, appname):
        self.appname = appname
        self.advisory_list = []

    def get_advisories(self, package, ecosystem):
        vulndata = []
        hasnext = True
        nextcursor = None
        query = yaml.safe_load(open(PATH_PROJECT + '/data/queries.yaml')).get(ecosystem)
        while hasnext:
            variables = {'package': package, 'after': nextcursor}
            request = requests.post('https://api.github.com/graphql',
                                    json={'query': query, 'variables': variables}, headers=headers)
            if request.status_code == 200:
                data = json.loads(request.content)
                vulndata = vulndata + data.get('data').get('securityVulnerabilities').get('nodes')
                hasnext = data.get('data').get('securityVulnerabilities').get('pageInfo').get('hasNextPage')
                nextcursor = data.get('data').get('securityVulnerabilities').get('pageInfo').get('endCursor')
            else:
                raise Exception("Error running the query. Error {}".format(request.status_code))

        return vulndata

    def validate_vulnerable_version(self, advisories, package, version):
        for advisory in advisories:
            major_eq = False
            minor_eq = False
            try:
                bottom, top = advisory.get('vulnerableVersionRange').split(',')
                if '>=' in bottom:
                    major_eq = True
                if '<=' in top:
                    minor_eq = True
                bottomnumber = bottom.replace('>', '').replace('=', '').strip()
                topnumber = top.replace('<', '').replace('=', '').strip()
                if bottomnumber < version < topnumber or (version == bottomnumber and minor_eq) or (
                        version == topnumber and major_eq):
                    advisory = self.parse_identifiers(advisory)
                    self.advisory_list.append({'package': package + ':' + version, 'advisory': advisory.copy()})
            except ValueError:  # no top version
                top = advisory.get('vulnerableVersionRange')
                if '<=' in top:
                    minor_eq = True
                topnumber = top.replace('<', '').replace('=', '').strip()
                if version < topnumber or (version == topnumber and minor_eq):
                    advisory = self.parse_identifiers(advisory)
                    self.advisory_list.append({'package': package + ':' + version, 'advisory': advisory.copy()})

    def parse_identifiers(self, advisory):
        dicts = advisory.get('advisory').get('identifiers')
        advisory.get('advisory')['cve'] = next((id for id in dicts if id.get('type') == "CVE"),
                                               {'type': 'CVE', 'value': ''})
        advisory.get('advisory')['ghsa'] = next((id for id in dicts if id.get('type') == "GHSA"),
                                                {'type': 'GHSA', 'value': ''})
        return advisory



import subprocess  # nosec
import sys
import os

PATH_PROJECT = os.path.realpath(os.path.dirname(__file__))
DEP_FILE = PATH_PROJECT + '/zasca_dep_tree.txt'


def generate_tree(filepath, include_dev):
    scope = '-Dscope=compile'
    exitcode = 0
    try:
        if include_dev:
            scope = '-Dscope=test'
        exitcode = subprocess.call(['mvn', 'dependency:tree', '-Doutput={}'.format(DEP_FILE), '-f',  # nosec
                                    filepath, scope],
                                   stdout=subprocess.DEVNULL,
                                   stderr=subprocess.STDOUT)
    except FileNotFoundError:
        print("Looks like maven is not correctly installed or the pom.xml file can't be resolved. Please check")
    if exitcode != 0:
        print("There has been an error building the tree")
        sys.exit(exitcode)
    else:
        return exitcode


def get_dependencies():
    dependencies = []
    dependency_info = {}
    with open(DEP_FILE) as f:
        data = f.read().replace('+-', '').replace('|', '').replace('\-', ''). \
            replace(' ', '').rstrip().split('\n')  # noqa: W605
    for dependency in data:
        dependency = dependency.replace('\n', '').split(':')
        if len(dependency) == 5 or len(dependency) == 4:
            dependency.pop(2)
        dependency_info['package'] = dependency[0] + ':' + dependency[1]
        dependency_info['group'] = dependency[0]
        dependency_info['name'] = dependency[1]
        dependency_info['version'] = dependency[2]
        dependencies.append(dependency_info.copy())
    appname = dependencies[0]
    del dependencies[0]
    os.remove(DEP_FILE)
    return dependencies, appname



from setuptools import setup, find_packages

with open('requirements.txt') as f:
    requirements = f.read()

__version__ = "2.0.0"

setup(
    name="zasca",
    version=__version__,
    description="Yet Another SCA tool, but with Z",
    author="Javier Dominguez",
    packages=find_packages(),
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'zasca=zasca.main:run_cli',
        ],
    },
    install_requires=requirements,
)



import json
from operator import itemgetter
import os
from jinja2 import Template
from datetime import datetime

PATH_PROJECT = os.path.realpath(os.path.dirname(__file__))
SEVERITY = {'CRITICAL': 4, 'HIGH': 3, 'MODERATE': 2, 'LOW': 1, 'OFF': 5}


def check_quality_gate(severity_data, threshold):
    qg_passed = True
    try:
        threshold_value = SEVERITY.get(threshold)
        for k, v in severity_data.items():
            if SEVERITY.get(k) >= threshold_value:
                qg_passed = False
        return qg_passed
    except TypeError:
        print("Quality gate not valid. It has to be CRITICAL, HIGH, MODERATE or LOW")
        return False


def suppress_fp(report, suppression_file):
    suppressed_issues = []
    try:
        with open(suppression_file) as file:
            fp_list = json.loads(file.read())
        for fp in fp_list:
            for vuln in report:
                advisory_list = vuln.get('advisory').get('advisory').get('identifiers')
                if vuln['package'] == fp.get('package') and fp.get('vulnerability') in map(itemgetter('value'),
                                                                                           advisory_list):
                    report.remove(vuln)
                    suppressed_issues.append(vuln.copy())
    except FileNotFoundError:
        print("File not found. Skipping suppression process..")
    except json.decoder.JSONDecodeError:
        print("JSON malformed. Skipping suppression process..")
    return report, suppressed_issues


def generate_cyclonedx_sbom(dependencies):
    with open(PATH_PROJECT + '/template/cyclonedx_template.json') as file:
        cyclone_data = json.loads(file.read())

    for item in dependencies:
        del item['package']
        item['type'] = 'library'
        cyclone_data['components'].append(item.copy())

    with open('cyclonedx_report.json', 'w') as file:
        file.write(json.dumps(cyclone_data))


def generate_html_report(data, appname):
    with open(PATH_PROJECT + '/template/report_template.html') as file:
        template = Template(file.read())
        template.globals['now'] = datetime.now().strftime("%d-%m-%Y %H:%M:%S")
    with open('sca_report.html', 'w') as file:
        file.write(template.render(report=data, appname=appname))
